@@ -16,6 +16,13 @@ from .util.spectral_object import Spectral_object
 
 class Cia_table(Spectral_object):
     """A class to handle CIA opacity data tables.
+
+    .. important::
+        In the 2018 release of Hitran, some CIA files can contain data for multiple
+        wavenumber range but for different temperatures in each range.
+        This is not yet supported so only the first wavenumber range is used.
+        For H2-H2 cia files, one should manually replace all the `n-H2 -- n-H2` in 
+        `H2-H2` for the format to be readable. 
     """
 
     def __init__(self, *filename_filters, filename=None, molecule_pair=None, search_path=None,
@@ -112,27 +119,44 @@ class Cia_table(Spectral_object):
                     Nw, Temp = self._read_header(file)
                 except EndOfFile:
                     break
-                tmp_tgrid.append(Temp)
-                tmp_abs_coeff.append([])
-                tmp_wns=[]
+                except:
+                    print("""
+                    The .cia format your are using must not be supported.
+                    This can happen with the 2018 release of the hitran CIA coefficients.
+                    In particular, for some couple of molecules, the name is written with
+                    spaces but exo_k only recognizes formats without spaces for now
+                    (for example: \'n-H2 -- n-H2\' should be \'H2-H2\').
+                    You can try replacing these in the files or contact me.
+                    """)
+                    raise RuntimeError('Bad .cia file format.')
+                tmp_abs_coeff2 = list()
                 if First: 
+                    tmp_wns=[]
                     for _ in range(Nw): #as iterator not used, can be replaced by _
                         line=file.readline()
                         tmp=line.split()
                         tmp_wns.append(float(tmp[0]))
-                        tmp_abs_coeff[-1].append(float(tmp[1]))
+                        tmp_abs_coeff2.append(float(tmp[1]))
                     self.wns=np.array(tmp_wns)
+                    self.Nw=self.wns.size
+                    First = False
                 else:
+                    if Nw != self.Nw:
+                        break # in 2018 format, there can be data for several
+                              # wavenumber range in the same file
+                              # but not for the same temperatures.
+                              # We take only the first set for now.
                     for _ in range(Nw):
                         line=file.readline()
                         tmp=line.split()
-                        tmp_abs_coeff[-1].append(float(tmp[1]))
+                        tmp_abs_coeff2.append(float(tmp[1]))
+                tmp_abs_coeff.append(tmp_abs_coeff2)
+                tmp_tgrid.append(Temp)
         self.wnedges=np.concatenate(([self.wns[0]],0.5*(self.wns[1:]+self.wns[:-1]),[self.wns[-1]]))
         self.tgrid=np.array(tmp_tgrid)
         self.abs_coeff=np.array(tmp_abs_coeff)
         self.abs_coeff_unit=old_cia_unit
         self.Nt=self.tgrid.size
-        self.Nw=self.wns.size
 
 
     def _read_header(self, file):
@@ -193,7 +217,35 @@ class Cia_table(Spectral_object):
         f.create_dataset("abs_coeff", data=self.abs_coeff,compression="gzip")
         f["abs_coeff"].attrs["units"] = self.abs_coeff_unit
         f.create_dataset("cia_pair", data=self.mol1+'-'+self.mol2)
-        f.close()    
+        f.close() 
+
+    def write_cia(self, filename):
+        """Writes simplified .cia files.
+
+        To be consistent with Hitran CIA files, data is converted back to cm^5
+        before writing.
+
+        Parameters
+        ----------
+            filename: str
+                Name of the file to be written.
+        """
+        if not filename.lower().endswith(('.cia')):
+            filename=filename+'.cia'
+        _,conversion_factor=unit_convert( \
+            'abs_coeff_unit',unit_file=self.abs_coeff_unit,
+            unit_in=self.abs_coeff_unit,unit_out='cm^5')
+        to_write=conversion_factor*self.abs_coeff
+        with open(filename, "w") as f:
+            for i_t, t in enumerate(self.tgrid):
+                header="""       {m1}-{m2}   {w1}  {w2}  {Nw}  {T}\n""".format(
+                    m1=self.mol1, m2=self.mol2, w1=self.wns[0],
+                    w2=self.wns[-1], Nw=self.Nw, T=t)
+                f.write(header) 
+                for i_w, wn in enumerate(self.wns):
+                    line=str(wn)+' '+str(to_write[i_t,i_w])+'\n'
+                    f.write(line) 
+   
 
     def sample(self, wngrid, remove_zeros=False, use_grid_filter=False, **kwargs):
         """Method to re sample a cia table to a new grid of wavenumbers (inplace).
@@ -369,24 +421,27 @@ class Cia_table(Spectral_object):
             'abs_coeff_unit',unit_file=tmp_k_u_file,unit_in=tmp_k_u_in,unit_out=tmp_k_u_out)
         self.abs_coeff=self.abs_coeff*conversion_factor
 
-    def convert_to_mks(self):
+    def convert_to_mks(self, verbose=False):
         """Converts units to MKS
         """
+        message=None
         if self.abs_coeff_unit=='cm^5':
-            print('Conversion from cm^5 to m^5')
+            message='Conversion from cm^5 to m^5'
             self.convert_abs_coeff_unit(abs_coeff_unit='m^5')
         elif self.abs_coeff_unit in ['cm^2','m^2']:
-            print('Conversion from '+self.abs_coeff_unit+'/amagat to m^5')
+            message='Conversion from '+self.abs_coeff_unit+'/amagat to m^5'
             self.convert_abs_coeff_unit(abs_coeff_unit='m^2')
             self.abs_coeff=self.abs_coeff*(KBOLTZ*273.15/101325.0)
             #conversion from m^2/amagat to m^5
         elif self.abs_coeff_unit in ['cm^-1','m^-1']:
-            print('Conversion from '+self.abs_coeff_unit+'/amagat^2 to m^5')
+            message='Conversion from '+self.abs_coeff_unit+'/amagat^2 to m^5'
             self.convert_abs_coeff_unit(abs_coeff_unit='m^-1')
             self.abs_coeff=self.abs_coeff*(KBOLTZ*273.15/101325.0)**2
             #conversion from m^-1/amagat^2 to m^5
         else:
-            pass
+            return
+        if verbose and message is not None:
+            print(message)
         self.abs_coeff_unit='m^5'
         return
 
