@@ -170,7 +170,7 @@ class Data_table(Spectral_object):
         self.convert_p_unit(p_unit='Pa')
 
     def interpolate_kdata(self, logp_array=None, t_array=None, x_array=1.,
-            log_interp=None, logp_interp=True, wngrid_limit=None):
+            log_interp=None, logp_interp=True, wngrid_limit=None, **kwargs):
         """interpolate_kdata interpolates the kdata at on a given temperature and
         log pressure profile. If a volume mixing ratio profile (`x_array`) is given,
         the cross section computed for the species is multiplied by `x_array` to account for the
@@ -180,7 +180,7 @@ class Data_table(Spectral_object):
         ----------
             logp_array: array
                 log 10 pressure array to interpolate to
-            t_array: array, same size a logp_array
+            t_array: array, same size as logp_array
                 Temperature array to interpolate to
             x_array: None
                 Volume mixing ratio array used to renormalize the cross section.
@@ -286,11 +286,11 @@ class Data_table(Spectral_object):
         kc_p1t0=self.kdata[lpindextended,tind-1]
         kc_p0t0=self.kdata[lpindextended-1,tind-1]
         if self._settings._log_interp is True:
-            kdata_tmp=  np.log10(kc_p1t1)*pw*tw          \
-                            +np.log10(kc_p0t1)*(1.-pw)*tw \
-                            +np.log10(kc_p1t0)*pw*(1.-tw) \
-                            +np.log10(kc_p0t0)*(1.-pw)*(1.-tw)
-            self.kdata=10**kdata_tmp
+            kdata_tmp=  np.log(kc_p1t1)*pw*tw          \
+                            +np.log(kc_p0t1)*(1.-pw)*tw \
+                            +np.log(kc_p1t0)*pw*(1.-tw) \
+                            +np.log(kc_p0t0)*(1.-pw)*(1.-tw)
+            self.kdata=np.exp(kdata_tmp)
         else:
             kdata_tmp=  (kc_p1t1)*pw*tw          \
                         +(kc_p0t1)*(1.-pw)*tw \
@@ -475,6 +475,27 @@ class Data_table(Spectral_object):
         """
         return self.kdata[key]
 
+    def set_kdata(self, new_kdata):
+        """Changes kdata (inplace). this is preferred to directly accessing kdata because this
+        method checks that the array has the right dimensions
+
+        Parameters
+        ----------
+            new_kdata: array
+                New array of kdata.
+        """
+        if not isinstance(new_kdata,np.ndarray):
+            new_kdata=np.array(new_kdata)
+        sh=np.array(new_kdata.shape)
+        if sh.size != self.shape.size:
+            raise RuntimeError('new_kdata does not have the right number of dimensions')
+        if np.all(sh == self.shape):
+            self.kdata=new_kdata
+        else:
+            print('Expected shape    : ', self.shape)
+            print('shape of new_kdata: ', sh)
+            raise RuntimeError('new_kdata does not have the right shape')
+
     def clip_spectral_range(self, wn_range=None, wl_range=None):
         """Limits the data to the provided spectral range (inplace):
 
@@ -600,6 +621,13 @@ class Data_table(Spectral_object):
         res.bin_down(wnedges=wnedges, **kwargs)
         return res
 
+    @property
+    def shape(self):
+        """Returns the shape that self.kdata should have to be compatible with
+        the parameter grid.
+        """
+        return np.array(list(filter(None, [self.Np,self.Nt,self.Nx,self.Nw,self.Ng])))
+
     def change_molecule_name(self, mol_name):
         """Changes name of the molecule (self.mol attribute).
 
