@@ -36,6 +36,36 @@ def bilinear_interpolation(z00, z10, z01, z11, x, y):
         res[i] = (z11[i]-z01[i]+z00[i]-z10[i])*xy +(z01[i]-z00[i])*y +(z10[i]-z00[i])*x +z00[i]
     return res
 
+@numba.njit(nogil=True,fastmath=True)
+def bilinear_interpolation_array(z00, z10, z01, z11, x, y):
+    """
+    2D interpolation
+    Applies linear interpolation across x and y between xmin,xmax and ymin,ymax.
+    For this variant of bilinear_interpolation (mainly for chemistry),
+    x, y and z must be 1D-arrays of the same length.
+
+    Parameters
+    ----------
+        z00: array
+            Array corresponding to xmin,ymin
+        z10: array
+            Array corresponding to xmax,ymin
+        z01: array
+            Array corresponding to xmin,ymax
+        z11: array
+            Array corresponding to xmax,ymax
+        x: array
+            weights on x coord
+        y: array
+            weights on y coord
+    """
+    xy = x*y
+    res = np.zeros_like(z00)
+    for i in range(z00.shape[0]):
+        res[i] = (z11[i]-z01[i]+z00[i]-z10[i])*xy[i] +(z01[i]-z00[i])*y[i] +(z10[i]-z00[i])*x[i] +z00[i]
+    return res
+
+
 @numba.njit(nogil=True,fastmath=True)
 def linear_interpolation(z00, z10, x):
     """1D interpolation.
@@ -206,7 +236,7 @@ def g_sample_5d(new_ggrid, new_kdata, old_ggrid, old_kdata):
             new_kdata[iP,iT,iX,iW,:]=np.interp(new_ggrid, old_ggrid, old_kdata[iP,iT,iX,iW,:])
     return
 
-#@numba.njit(nogil=True,fastmath=True)
+@numba.njit(nogil=True,fastmath=True)
 def RandOverlap_2_kdata_prof(Nlay, Nw, Ng, kdata1, kdata2, weights, ggrid):
     """Function to randomely mix the opacities of 2 species in an atmospheric profile.
 
@@ -238,6 +268,13 @@ def RandOverlap_2_kdata_prof(Nlay, Nw, Ng, kdata1, kdata2, weights, ggrid):
 
     for ilay in range(Nlay):
         for iW in range(Nw):
+            if (kdata1[ilay,iW,-1]-kdata2[ilay,iW,0])*(kdata2[ilay,iW,-1]-kdata1[ilay,iW,0])<0.:
+                #ii+=1
+                if kdata1[ilay,iW,-1] > kdata2[ilay,iW,-1]:
+                    newkdata[ilay,iW,:] = kdata1[ilay,iW,:] 
+                else:
+                    newkdata[ilay,iW,:] = kdata2[ilay,iW,:]
+                continue
             tmp=kdataconv[ilay,iW,:]
             indices=np.argsort(tmp)
             kdatasort=tmp[indices]
